package com.game.business.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.game.business.dto.AppUserDividendListDto;
import com.game.business.dto.AppUserDividendSendDto;
import com.game.common.core.domain.HttpRet;
import com.game.common.core.domain.HttpRetPageArr;
import com.game.common.core.domain.R;
import com.game.common.utils.SecurityUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.game.common.annotation.Log;
import com.game.common.core.controller.BaseController;
import com.game.common.core.domain.AjaxResult;
import com.game.common.enums.BusinessType;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.game.business.domain.AppUserCountDividend;
import com.game.business.service.IAppUserCountDividendService;
import com.game.common.utils.poi.ExcelUtil;
import com.game.common.core.page.TableDataInfo;

/**
 * 用户代理分红Controller
 * 
 * @author game
 * @date 2024-06-30
 */
@RestController
@RequestMapping("/business/dividend")
@Api(value = "AppUserCountDividendController", description = "用户代理表接口", tags = {"用户代理表接口"})
public class AppUserCountDividendController extends BaseController
{
    @Autowired
    private IAppUserCountDividendService appUserCountDividendService;

    /**
     * 查询用户代理分红列表
     */
    @GetMapping("/list")
    @ApiOperation(value = "查询周分红列表", notes = "获取周分红列表")
    public HttpRetPageArr<AppUserCountDividend> list(AppUserDividendListDto dto)
    {
        startPage();
        List<AppUserCountDividend> list = appUserCountDividendService.selectUserDividendList(SecurityUtils.getUserId());
        return HttpRetPageArr.success("查询成功", getDataTable(list));
    }

    /**
     * 查询用户代理分红列表
     */
    @GetMapping("/send")
    @ApiOperation(value = "发放分红", notes = "发放分红")
    public HttpRet<Boolean> send(AppUserDividendSendDto dto)
    {

        return appUserCountDividendService.send(dto.getUserId(),dto.getDateNum());
    }

}
